import pandas as pd
oheMap_encode ,oheMap_decode  = {} , {}

def encode(df , cat_cols):
    for col in cat_cols:
        vals = df[col].value_counts()
        vals = [(freq, char) for char, freq in vals.items()]
        vals.sort()
        oheMap_encode[col] = {}
        oheMap_decode[col] = {}
        for i in range(len(vals)):
            char = vals[i][1]
            oheMap_encode[col][char] = i+1
            oheMap_decode[col][i+1] = char
        
        df[col] = df[col].apply(lambda x: oheMap_encode[col][x] if not pd.isna(x) else x)




def decode(df , cat_cols):
    for col in cat_cols:
        df[col] = df[col].apply(lambda x: oheMap_decode[col][x] if not pd.isna(x) else x)
        



from typing import Literal
import numpy as np
from clustered_imputation.clusterBase import cluster_features


def getClusters(df, num_imputation: Literal["mean", "median"], corr_threshold : 0.6 , num_cols , cat_cols):
    df_copy = df
    for col in num_cols:
        if num_imputation == "mean":
            x = df_copy[col].mean()
        else:
            x = df_copy[col].median()
        df_copy[col] = df_copy[col].fillna(x)
    for col in cat_cols:
        x = df_copy[col].mode()[0]
        df_copy[col] = df_copy[col].fillna(x)
    clusters = cluster_features(df_copy, corr_threshold=corr_threshold)
    finalClusters = []
    single = []
    for cluster in clusters:
        if len(cluster) > 1:
            finalClusters.append(cluster)
        else:
            single.append(cluster[0])
    finalClusters.append(single)
    return finalClusters



import pandas as pd
import numpy as np

# Define categories
categories1 = ['A', 'B', 'C']
categories2 = ['X', 'Y', 'Z']
categories3 = ['Red', 'Green', 'Blue']

# Create DataFrame
np.random.seed(42)
data = pd.DataFrame({
    'Cat1': np.random.choice(categories1, 20),
    'Cat2': np.random.choice(categories2, 20),
    'Cat3': np.random.choice(categories3, 20),
    'Num1': np.random.randint(10, 100, 20),
    'Num2': np.random.uniform(1.0, 50.0, 20),
    'Num3': np.random.randint(200, 500, 20)
})

# Introduce missing values
for col in data.columns:
    data.loc[np.random.choice(data.index, size=5, replace=False), col] = np.nan



def cluster_features(df, corr_threshold = 0.6):
    corr_matrix = df.corr().abs()  # absolute correlation matrix
    clusters = []  # list of clusters
    visited = set()

    for col in corr_matrix.columns:
        if col in visited:
            continue
        # Start a new cluster and add the current column if it has correlated columns
        cluster = [col]
        for other_col in corr_matrix.columns:
            if other_col not in visited and other_col != col and corr_matrix[col][other_col] > corr_threshold:
                cluster.append(other_col)
                visited.add(other_col)
        visited.add(col)  # Mark the current column as visited
        clusters.append(cluster)  # Only add non-empty clusters

    #print(clusters)
    return clusters